@@ -896,22 +896,22 @@ int __get_consecutive_ports(GQueue *out, unsigned int num_ports, unsigned int wa
 		port += PORT_RANDOM_MIN + (ssl_random() % (PORT_RANDOM_MAX - PORT_RANDOM_MIN));
 #endif
 		__C_DBG("after  randomization port=%d", port);
-	}
 
-	// debug msg if port is in the given interval
-	if (bit_array_isset(pp->ports_used, port)) {
-		__C_DBG("port %d is USED in port pool", port);
-		mutex_lock(&pp->free_list_lock);
-		unsigned int fport = GPOINTER_TO_UINT(g_queue_pop_head(&pp->free_list));
-		if (fport)
-			bit_array_clear(pp->free_list_used, fport);
-		mutex_unlock(&pp->free_list_lock);
-		if (fport) {
-			port = fport;
-			__C_DBG("Picked port %u from free list", port);
+		// debug msg if port is in the given interval
+		if (bit_array_isset(pp->ports_used, port)) {
+			__C_DBG("port %d is USED in port pool", port);
+			mutex_lock(&pp->free_list_lock);
+			unsigned int fport = GPOINTER_TO_UINT(g_queue_pop_head(&pp->free_list));
+			if (fport)
+				bit_array_clear(pp->free_list_used, fport);
+			mutex_unlock(&pp->free_list_lock);
+			if (fport) {
+				port = fport;
+				__C_DBG("Picked port %u from free list", port);
+			}
+		} else {
+			__C_DBG("port %d is NOT USED in port pool", port);
 		}
-	} else {
-		__C_DBG("port %d is NOT USED in port pool", port);
 	}
 
 	while (1) {
@@ -1230,6 +1230,7 @@ static const char *kernelize_one(struct rtpengine_target_info *reti, GQueue *out
 	if (!reti->decrypt.cipher || !reti->decrypt.hmac)
 		return "decryption cipher or HMAC not supported by kernel module";
 
+	reti->track_ssrc = 1;
 	for (unsigned int u = 0; u < G_N_ELEMENTS(stream->ssrc_in); u++) {
 		if (stream->ssrc_in[u])
 			reti->ssrc[u] = htonl(stream->ssrc_in[u]->parent->h.ssrc);
@@ -1407,6 +1408,7 @@ void kernelize(struct packet_stream *stream) {
 		g_slice_free1(sizeof(*redi), redi);
 	}
 
+	stream->kernel_time = rtpe_now.tv_sec;
 	PS_SET(stream, KERNELIZED);
 	return;
 
@@ -1414,6 +1416,7 @@ no_kernel_warn:
 	ilog(LOG_WARNING, "No support for kernel packet forwarding available (%s)", nk_warn_msg);
 no_kernel:
 	PS_SET(stream, KERNELIZED);
+	stream->kernel_time = rtpe_now.tv_sec;
 	PS_SET(stream, NO_KERNEL_SUPPORT);
 }
 
@@ -1684,9 +1687,9 @@ static bool __stream_ssrc_inout(struct packet_stream *ps, uint32_t ssrc, mutex_t
 	int ctx_idx = __hunt_ssrc_ctx_idx(ssrc, list, 0);
 	if (ctx_idx == -1) {
 		// SSRC mismatch - get the new entry:
+		ctx_idx = *ctx_idx_p;
 		// move to next slot
-		ctx_idx = (*ctx_idx_p + 1) % RTPE_NUM_SSRC_TRACKING;
-		*ctx_idx_p = ctx_idx;
+		*ctx_idx_p = (*ctx_idx_p + 1) % RTPE_NUM_SSRC_TRACKING;
 		// eject old entry if present
 		if (list[ctx_idx])
 			ssrc_ctx_put(&list[ctx_idx]);
@@ -1865,8 +1868,6 @@ static void media_packet_rtp_in(struct packet_handler_ctx *phc)
 	bool unkern = false;
 
 	if (G_LIKELY(!phc->rtcp && !rtp_payload(&phc->mp.rtp, &phc->mp.payload, &phc->s))) {
-		rtp_padding(phc->mp.rtp, &phc->mp.payload);
-
 		unkern = __stream_ssrc_in(phc->in_srtp, phc->mp.rtp->ssrc, &phc->mp.ssrc_in,
 				phc->mp.media->monologue->ssrc_hash);
 
@@ -2405,6 +2406,8 @@ static int stream_packet(struct packet_handler_ctx *phc) {
 	if (handler_ret < 0)
 		goto out; // receive error
 
+	rtp_padding(phc->mp.rtp, &phc->mp.payload);
+
 	// If recording pcap dumper is set, then we record the call.
 	if (phc->mp.call->recording)
 		dump_packet(&phc->mp, &phc->s);
@@ -2421,7 +2424,8 @@ static int stream_packet(struct packet_handler_ctx *phc) {
 	RTPE_STATS_INC(packets_user);
 	RTPE_STATS_ADD(bytes_user, phc->s.len);
 
-	count_stream_stats_userspace(phc->mp.stream);
+	if (!PS_ISSET(phc->mp.stream, KERNELIZED) || rtpe_now.tv_sec > phc->mp.stream->kernel_time + 1)
+		count_stream_stats_userspace(phc->mp.stream);
 
 	int address_check = media_packet_address_check(phc);
 	if (address_check)
